#!/usr/bin/perl -w

# Script for installing FreeVSD following compilation from source.
# (RedHat GNU Linux version)
# Copyright (c) 1999, 2000 Idaya Ltd.
#
# This file is part of the Virtual Server Administrator (FreeVSD)
#
# FreeVSD is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# FreeVSD is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with FreeVSD; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#

use VSD;
use VSD::Change;
use VSD::Config;
use Fcntl;
use strict;



# Ensure we are only run once...

system '/bin/grep -q "vsd 1725/tcp" /etc/services' or error ( 'installed' );



# Globals...

# %Path imported from VSD.pm

my $sbindir="$path{vsdsbindir}";
my $vsdconf="$path{vsdconf}";

my $resolv = "/etc/resolv.conf";

my $inetd_conf = '/etc/inetd.conf';
my $xinetd_conf = '/etc/xinetd.conf';

-e $inetd_conf || undef $inetd_conf; -e $xinetd_conf || undef $xinetd_conf;


my $localhost = `hostname`;
chomp $localhost;
$localhost =~ s/^\s*(\S+)\s*$/$1/;


# Create new object to log all changes made in this script.
#   Log file is $VSD::path{vsdconfdir}/undo/vsd.undo

my $change = VSD::Change->new( file => $vsdconf );





# ------------------------------- vsd confguration changes


# Get mount point for skel and virtual servers...

print ("\nEnter mount point for FreeVSD skel and virtual servers [/home/vsd] : ");
my $mntpoint = <>;
$mntpoint = "/home/vsd" if (length $mntpoint == 1);




# Get name server ips...

my ( $pnsip, $snsip ) = nss( $resolv );
defined $pnsip || ($pnsip = ''); defined $snsip || ($snsip = '');
print ("\nEnter IP address of primary name server [$pnsip] : ");
my $pns = <>;
chomp $pns;
$pns = $pnsip if ($pns eq "");
print ("\nEnter IP address of secondary name server [$snsip] : ");
my $sns = <>;
chomp $sns;
$sns = $snsip if ($sns eq "");


$pns || do {
    print "\n\nWARNING: No primary name server specified.\n";
    print "                               Abort installation ? [Y] : ";
    (<>) =~ /n/i || die "\nInstallation aborted, no changes made.\n\n"
};




# Modify configuration file...

print "\nCustomising configuration file...\n";
print " Configured the following lines in $vsdconf:\n";

# Pass a reference to the VSD::Change object to VSD::Config,
#  so it knows where the logging is going on.
#  (And we're not talking about Canada)
my $vsdconftmp = VSD::Config->new ( change => $change );

$pns && do { $vsdconftmp->setval ( 'Global', 'PrimaryNS', $pns );
	     print "  PrimaryNS $pns\n" } ||
  $vsdconftmp->delval ( 'Global', 'PrimaryNS' );

$sns && do { $vsdconftmp->setval ( 'Global', 'SecondaryNS', $sns );
	     print "  SecondaryNS $sns\n" } ||
  $vsdconftmp->delval ( 'Global', 'SecondaryNS' );

$vsdconftmp->setval ( 'Partition', '0', 'Mount', $mntpoint );
print "  Mount $mntpoint\n";

if ( -d "$mntpoint/skel" and opendir SKEL, "$mntpoint/skel" ) {
    my @skels = readdir SKEL;
    closedir SKEL;
    @skels > 2 && do {
	print "\n\n  VSD has detected the presence of ".
	  (@skels>3 ? 'skels' : 'a skel')." in $mntpoint/skel\n";
	print "\n     NOTE: If you already have ".
	  (@skels>3 ? 'skels' : 'a skel').", you do";
	print "\n            not need to run vsd-genskel.pl, unless";
	print "\n            you want to create a new skel or replace";
	print "\n            an existing one.\n";
	print "\n  WARNING: Only register (ie. add to vsd.conf) skels ";
	print "\n            that you can be sure are not corrupt.\n";
	foreach my $skel ( @skels ) {
	    $skel !~ /^\.{1,2}$/ && -d "$mntpoint/skel/$skel" && do {
		print "\n   Do you want to register the skel named $skel ? [N]: ";
		(<>) =~ /y/i
		  && ($vsdconftmp->setval ( 'Partition', '0', 'Skel', $skel ))
		    || do {
			print "\n    Remove this skel ? [N]: ";
			(<>) =~ /y/i && do {
			    print "\n     Removing $skel ...";
			    `rm -rf $mntpoint/skel/$skel`;
			    print "...removed.\n";
			};
		    };
	    };
	}
    };
    @skels = $vsdconftmp->getval ( 'Partition', '0', 'Skel' );
    @skels && do {
    print "\n  Do you want to set a default skel for the";
    print "\n   creation of virtual servers ? [Y]: ";
    (<>) =~ /n/i
      || do {
	  print "\n     NOTE: If you don't set a default skel ";
	  print "\n            now, you can do it using vsd-genskel.pl later.\n";
	  print "\n   Skels available :\n";
	  print "\n     @skels\n";
	  print "\n   Enter your chosen skel's name,";
	  print "\n    or hit return to continue without setting a default skel : ";
	  my $default_skel = <>;
	  if ( $default_skel !~ /^\s*$/ ) {
	      $default_skel =~ s/^\s*(\S+)\s*$/$1/;
	      if ( grep /^$default_skel$/, @skels ) {
		  print "\n   Setting default skel to $default_skel\n";
		  $vsdconftmp->setval ( 'Global', 'Skel', $default_skel );
	      }
	      else { print "\n   Unknown skel. No default skel set.\n" }
	  }
	  else { print "\n   No default skel set.\n" }
      };
   };
}


# Set up loopback VirtualServer to point to localhost...

print "\nSetting up VirtualServer $localhost, for loopback,";
print "\n in vsd.conf and linking it to / ...\n";

my $ip = `hostname -i $localhost`;
chomp $ip;
$ip =~ s/^\s*(\S+)\s*$/$1/;

$vsdconftmp->setval ( 'VirtualServer', $localhost, 'IP', $ip );
$vsdconftmp->setval ( 'VirtualServer', $localhost, 'Status', 'active' );
$vsdconftmp->setval ( 'VirtualServer', $localhost, 'Partition', '0' );
# FIX put in 'HostingServer' entry here ...
# Skel section already set up above, if chosen.

-d "$mntpoint" or mkdir $mntpoint, 0755 or die "Can't create dir $mntpoint";
-d "$mntpoint/vs" or mkdir "$mntpoint/vs", 0755 or die "Can't create dir $mntpoint/vs";

# FIX $Change......command.

  $change->command (
		    "ln -s / $mntpoint/vs/$localhost",
		    "rm -f $mntpoint/vs/$localhost",
		    "unlink / from $mntpoint/vs/$localhost ( you really do b.t.w )"
		   );
#symlink '/', "$mntpoint/vs/$localhost" or
#  die "\n  Unable to create link from $mntpoint/vs/$localhost to /\n";





# ----------------------------- xinetd / inetd configuration changes


# inetd configuration...

if ( $inetd_conf ){

 # Virtualise relevant services...

    $change->file( $inetd_conf );

    print "\n\nVirtualising the following services (ftp, telnet, pop-3, smtp)...\n";
    print " hash commented the following lines in /etc/inetd.conf:\n";
    $change->comment (
		      'ftp',
		      'telnet',
		      'pop-3',
		      'smtp',
		      );

    system '/bin/grep -q "usr/sbin/proftpd" /etc/inetd.conf' && 
	$change->append ("ftp stream tcp nowait root $sbindir/virtuald tcpd  /usr/sbin/proftpd -d -l");
    system '/bin/grep -q "usr/sbin/in.telnetd" /etc/inetd.conf' &&
	$change->append ("telnet stream tcp nowait root $sbindir/virtuald tcpd /usr/sbin/in.telnetd");
    system '/bin/grep -q "usr/sbin/in.qpopper" /etc/inetd.conf' &&
	$change->append ("pop-3 stream tcp nowait root $sbindir/virtuald tcpd /usr/sbin/in.qpopper");
    system '/bin/grep -q "usr/sbin/sendmail" /etc/inetd.conf' && 
	$change->append ("smtp stream tcp nowait root $sbindir/virtuald tcpd /usr/sbin/sendmail -bs");

    print "\nAppended the following lines to /etc/inetd.conf:\n";
    print "  ftp stream tcp nowait root $sbindir/virtuald tcpd /usr/sbin/in.ftpd -d -l\n";
    print "  telnet stream tcp nowait root $sbindir/virtuald tcpd /usr/sbin/in.telnetd\n";
    print "  pop-3 stream tcp nowait root $sbindir/virtuald tcpd /usr/sbin/ipop3d\n";
    print "  smtp stream tcp nowait root $sbindir/virtuald tcpd /usr/sbin/sendmail -bs\n\n";

# Checking for and creating vsd entry in inetd configuration

    system "/bin/grep -q \"$sbindir/vsd\" $inetd_conf" &&
	$change->append( "vsd stream tcp nowait root $sbindir/virtuald tcpd $sbindir/vsd" );
    system "/bin/grep -q \"$sbindir/vsd\" $inetd_conf" &&
	$change->append( "svsd stream tcp nowait root $sbindir/virtuald tcpd $sbindir/vsd" );
    print "Appended the following lines to $inetd_conf:\n";
    print "  vsd stream tcp nowait root $sbindir/virtuald tcpd $sbindir/vsd\n";
    print "  svsd stream tcp nowait root $sbindir/virtuald tcpd $sbindir/vsd\n\n";

}



# xinetd configuration...

if ( $xinetd_conf ){

    $change->file( $xinetd_conf );

    # Virtualise services already existing in xinetd.conf...

    $change->xinetd_virtualise ();

    # Having virtualised existing services, check for a defaults section and
    #  for various useful utils and add them if they are not there.

  # FIX needs to be put into a conf module or VSD.pm, well, not in here anyway...

    my %utils  = (
	              defaults => [
				   "   only_from = 127.0.0.1\\32"
				  ],
			 
		      ftp      => [
				   "     socket_type     = stream",
				   "     protocol        = tcp",
				   "     wait            = no",
				   "     user            = root",
				   "     server          = $sbindir/virtuald",
				   "     server_args     = /usr/sbin/proftpd -d -l"
				  ],
		      telnet   => [
				   "",
				   "     socket_type     = stream",
				   "     protocol        = tcp",
				   "     wait            = no",
				   "     user            = root",
				   "     server          = $sbindir/virtuald",
				   "     server_args     = /usr/sbin/in.telnetd"
				  ],
		   'pop-3'    => [
				   "     socket_type     = stream",
				   "     protocol        = tcp",
				   "     wait            = no",
				   "     user            = root",
				   "     server          = $sbindir/virtuald",
				   "     server_args     = /usr/sbin/in.qpopper"
				  ],
		     smtp      => [
				   "     socket_type     = stream",
				   "     protocol        = tcp",
				   "     wait            = no",
				   "     user            = root",
				   "     server          = $sbindir/virtuald",
				   "     server_args     = /usr/sbin/sendmail -bs"
				  ],
		  vsd         => [
				  "     socket_type     = stream",
				  "     protocol        = tcp",
				  "     wait            = no",
				  "     user            = root",
				  "     server          = $sbindir/virtuald",
				  "     server_args     = $sbindir/vsd"
				 ]
		 );


  # Add if not already there...

    $change->xinetd_append ( \%utils );


    print "\nAppended the following services to $xinetd_conf, if not found:\n";
    print "  service ftp: /usr/sbin/proftpd -d -l\n";
    print "  service telnet: /usr/sbin/in.telnetd\n";
    print "  service pop-3: /usr/sbin/in.qpopper\n";
    print "  service smtp: /usr/sbin/sendmail -bs\n\n";
    print "  service vsd: $sbindir/vsd\n\n";

}




# ------------------------------------ register the vsd service


print "Registering the vsd service...\n";

$change->file( '/etc/services' );

system '/bin/grep -q "vsd" /etc/services' && $change->append ( 'vsd  1725/tcp  # Virtual Server Daemon (VSD)' );
system '/bin/grep -q "vsd" /etc/services' && $change->append ( 'svsd 1726/tcp  # Secure Virtual Server Daemon (VSD over SSL)' );

print "Appended the following lines to /etc/services:\n";
print "  vsd  1725/tcp  # Virtual Server Daemon (VSD)\n";
print "  svsd 1726/tcp  # Secure Virtual Server Daemon (VSD over SSL)\n\n";




# ------------------------------------ apply config changes


# Disable automatic startup of httpd service
#  and kill any instances which are running

$change->command( '/sbin/chkconfig httpd off', '/sbin/chkconfig httpd on', 'enable httpd at startup' );
$change->command( '/usr/bin/killall -q httpd', '/etc/rc.d/init.d/httpd start', 'start httpd' );

print "Disabled httpd at startup\nKilled all httpd processes\n\n";



# Sending HUP to inetd or xinetd

if (-e "/usr/sbin/inetd")
{
  print "Forcing inetd to read configuration changes...\n\n";
  $change->command (
		    '/usr/bin/killall -HUP inetd',
		    '/usr/bin/killall -HUP inetd',
		    're-read inetd config changes'
		   );
}

if (-e "/usr/sbin/xinetd")
{
  print "Forcing xinetd to read configuration changes...\n\n";
  $change->command (
		    '/etc/rc.d/init.d/xinetd restart',
		    '/etc/rc.d/init.d/xinetd restart',
		    're-read xinetd config changes'
		   );
}



# FIX put nameserver resolve in VSD::Config.pm

sub nss {
    open RESOLV, "< $_[0]" or die "error opening $_[0]: $!";
    my $match = 0;
    my @nss;
    while (defined ($_=<RESOLV>) && @nss<2) {
	/^nameserver\s*(\d+).(\d+).(\d+).(\d+)*/i && (push @nss, "$1.$2.$3.$4");
    }
    close RESOLV or die "error closing $_[0]: $!";
    return @nss;
}


sub error {
    my $error = shift;

    my %error;

    $error{installed} =<<ORP;
  FreeVSD appears to be installed already
   because your /etc/services file has a
   'vsd' entry.

  Run vsd-uninstall.pl and then run
   vsd-install.pl again.
   If the problem is still there, edit 
   /etc/services by hand to remove the
   'vsd ...' line. Then run vsd-install.pl
   once more.
ORP

  print "\nError:\n\n$error{$error}\n";
  die;
}










