#!/usr/bin/perl
# Script for uninstalling FreeVSD following compilation from source.
# (RedHat GNU Linux version)
# Copyright (c) 1999, 2000 Idaya Ltd.
#
# This file is part of the Virtual Server Administrator (FreeVSD)
#
# FreeVSD is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# FreeVSD is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with FreeVSD; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#

# use strict; # FIX and -w -- Still not ready for this yet until VSD.pm
#                              is strict too and everything else...

use VSD;
use VSD::Undo;
use VSD::Config;

my $vsdconfdir = "$path{vsdconfdir}";
my $vsdcertdir = "$vsdconfdir/ca";
my $vsdundodir = "$vsdconfdir/undo";


my $vsdconf = VSD::Config->new( file=>$path{vsdconf} );


my $localhost = `hostname`;
chomp $localhost;
$localhost =~ s/^\s*(\S+)\s*$/$1/;


#my $mntpoint = $vsdconf->getval(
#				'Partition',
#				'0',
#				'Mount'
#			       );
#  FIX soon not only Partition 0



my $vsd_install_undo = VSD::Undo->new( script => 'vsd-install.pl' );

defined $vsd_install_undo or die "Nothing to uninstall.";




# Turn off virtual servers..
system "$path{vsdsbindir}/vsboot --stop";




my %files = %{$vsd_install_undo->file_list()};
my %commands = %{$vsd_install_undo->command_list()};
my %command_comments = %{$vsd_install_undo->command_comments()};

foreach my $file ( keys %files ) {
    print "\nUndo changes to $file ? [Y] : ";
    if (confirmy()) {
	$vsd_install_undo->file( $file );
    }
}

# FIX needs ordering...

foreach my $command ( keys %commands ) {
    print "\n".(defined $command_comments{$command} ? 
		"Do you want to $command_comments{$command}" :
		"Undo command $command").' ? [Y] : ';
    if (confirmy()) {
	print "  Executing command: $commands{$command} ...\n";
	$vsd_install_undo->command( $command );
    }
}




# ------------------- vsd-genskel changes ------------------------------


my %partitions = $vsdconf->getval( 'Partition' );

foreach my $partition ( keys %partitions ) {

    print "\n\nPartition $partition ...\n";

    my $mount = $vsdconf->getval(
				 'Partition',
				 $partition,
				 'Mount'
				);
    my @skels = $vsdconf->getval(
				 'Partition',
				 $partition,
				 'Skel'
				);
    mount_clean ( $mount, \@skels );
}


# FIX new path stuff ..... ?
if (-d "$vsdcertdir") {
    print "\nRemove authorised certificates in $vsdcertdir? [N] : ";
    if (confirmn()) {
	`/bin/rm -rf $vsdcertdir`;
    }
}

if (-d "$vsdundodir") {
    print "\nRemove vsd undo directory and log, $vsdundodir? [N] : ";
    if (confirmn()) {
	`/bin/rm -rf $vsdundodir`;
    }
}



print "\n\n....finished.\n\n";






sub confirmy {
    $_ = <>;
    return 0 if (/^n.*/i);
    return 1;
}

sub confirmn {
    $_ = <>;
    return 1 if (/^y.*/i);
    return 0;
}


sub mount_clean {
    my $mount = $_[0];
    my @skels = @{$_[1]};

    # FIX when the ca stuff is ready...
    if (-d "$mount/ca") {
	print "\nRemove certificate authority $mount/ca? [N] : ";
	if (confirmn()) {
	    `/bin/rm -rf $mount/ca`;
	}
    }

    if (-d "$mount/certs") {
        print "\nRemove certificate directory $mount/certs? [N] : ";
        if (confirmn()) {
            `/bin/rm -rf $mount/certs`;
        }
    }

    my $rm_count = 0;
    foreach my $skel ( @skels ) {
	defined $skel || next;
	if (-d "$mount/skel/$skel" && $skel ne '') {
	    print "\n  Remove skel, $skel ? [N] : ";
	    confirmn() && do {
		`/bin/rm -rf $mount/skel/$skel`;
		$rm_count++
	    };
	}
	else {
	    print "\n  Note: Skel, $skel, in $path{vsdconf}, doesn't exist.\n";
	    $rm_count++
	}
    }

    if ( -d "$mount" ) {
	if ( $rm_count >= @skels ) {
	    print "\n    All skels now removed...\n";
	    print "\n  Remove mount point, $mount, ";
	    print "including all virtual servers beneath? [N] : ";
	    if (confirmn()) {
		-d "$mount/vs/$localhost" && `/bin/rm -f $mount/vs/$localhost`;
		`/bin/rm -rf $mount`;
	    }
	}
	else {
	    print "\n  Remove virtual servers beneath mount point, $mount ? [N] : ";
	    if (confirmn()) {
		-d "$mount/vs/$localhost" && `/bin/rm -f $mount/vs/$localhost`;
		`/bin/rm -rf $mount/vs`;
	    }	
	}
	
    }
}


