/* Interface to sending signals to BIND.
   This file is part of WebDNS. 
 */

/*
 * VSD - Virtual Server Daemon
 * Copyright (c) 2000 - 2001 Idaya Ltd.
 *
 * VSD is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * VSD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with VSD - see the COPYING file;  if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 * See the AUTHORS file for a list of contributors to VSD.
 * See the ChangeLog files for a list of changes.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>

#if 0
#include <sys/socket.h>

#include <netinet/in.h>
#include <arpa/nameser.h>
#include <arpa/inet.h>

#include <isc/eventlib.h>
#include <isc/ctl.h>
#endif

#include "vsd.h"
#include "bind.h"

#if 0
#define CHANNEL "/var/run/ndc"

struct args
{
  char *cmd;
  closure cl;
  void *ua;
}

static int doneflag = 0;

static void logger (enum ctl_severity ctlsev, const char *fmt, ...)
{
  va_list args;

  va_strat (args, format);
  if (ctlsev == ctl_warning)
    vsd_log_error (VSD_LOG_INFO, args);
  else if (ctlsev == ctl_error)
    vsd_log_error (VSD_LOG_ERROR, args);

  va_end (args);
}

static void done (struct ctl_cctx *ctl, void *uap, const char *msg,
		  u_int flags)
{
  struct args *a = uap;

  if (a->cl)
    (a->cl) (a->ua, msg, flags);
  if ((flags & CTL_MORE) == 0)
    doneflag = 1;
}

char *bind_named_notify_load (struct connection *vc, const char *domain)
{
  struct ctl_cctx *ctl;
  struct sockaddr *client_addr;
  struct args a;
  struct evContext ev;
  evEvent e;

  if (evCreate (&ev) < 0)
    return "cannot create event context";

  a.cmd = (char *) malloc (strlen (domain) + 15);
  sprintf (a.cmd, "reload %s", domain);
  a.cl = NULL;
  a.ua = NULL;

  ctl = ctl_client (ev, client_addr,
		    sizeof (struct sockaddr_un),
		    (struct sockaddr *) &server,
		    sizeof (struct sockaddr_in),
		    NULL, &a, 15, logger);
  if (ctl == NULL)
    return "cannot connect to command channel";
  else
    {
      doneflag = 0;
      while (evGetNext (ev, &e, EV_WAIT) == 0)
	if (evDispatch (ev, e) < 0 || doneflag)
	  break;
      ctl_endclient (ctl);
    }

  evDestroy (ev);

  return NULL;
}

void bind_named_notify_update (struct connection *vc, const char *domain)
{
}

#endif

/* If a domain has been added or deleted then call this function to
   tell named about it.  */
void bind_named_notify_load (struct connection *vc, const char *domain)
{
  char cmd[256];

  if (strlen (domain) >= sizeof (cmd))
    return;

#if 0
  sprintf (cmd, "reconfig %s", domain);
  run_command (vc, NULL, "root", "/usr/sbin/ndc", cmd, NULL, NULL);
#endif
}

/* If a domain has been modified then call this function to tell
   named about it.  */
void bind_named_notify_update (struct connection *vc, const char *domain)
{
  char cmd[256];

  if (strlen (domain) >= sizeof (cmd))
    return;

  sprintf (cmd, "reload %s", domain);
  run_command (vc, NULL, "root", "/usr/sbin/ndc", cmd, "/dev/null", "/dev/null");
}
