/* ca.c - ca module for openssl ca manipulation.
 */

/* 
 * VSD - Virtual Server Daemon
 * Copyright (c) 2000 - 2001 Idaya Ltd.
 *
 * VSD is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either 
 * version 2 of the License, or (at your option) any later version.
 *
 * VSD is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with VSD - see the COPYING file;  if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA.  
 */

/*
 * See the AUTHORS file for a list of contributors to VSD. 
 * See the ChangeLog files for a list of changes. 
 */

#include "config.h"

#include <stdio.h>
#include <unistd.h>

#include "vsd.h"
#include "libvsd.h"
#include "ca.h"

/********************************************************************************
 * Description: Add a VSD cert/key pair to VSD CA
 *
 *     Returns: void
 *******************************************************************************/
void 
ca_certadd (struct connection *vc, 
	    const int         argc, 
	    const char        *argv[])
{
  char *cmd, tmp[128];
  int x;

  if (argc != 1)
    {
      io_buffer_store (vc, "-ERR [ca] Incorrect arguments.\n");
      return;
    }
  
  memset (tmp, 0, sizeof(tmp));
#ifdef DEBUG
  srand (time (NULL));
  x = rand () % 1383252;
  sprintf (tmp, "/tmp/vsd%06u", x);
#else
  strcpy (tmp, "/dev/null");
#endif

  if (access ("/usr/sbin/vsd-ca_certadd", R_OK | X_OK) == 0)
    cmd = "/usr/sbin/vsd-ca_certadd";
  else if (access ("/usr/local/sbin/vsd-ca_certadd", R_OK | X_OK) == 0)
    cmd = "/usr/local/sbin/vsd-ca_certadd";
  else
    {
      io_buffer_store (vc, "-ERR [ca] Cannot find vsd-ca_certadd.\n");
      return;
    }

  if (run_command (vc, vc->server_root, "root", cmd, argv[0], NULL, tmp))
    io_buffer_store (vc, "-ERR [ca] vsd-ca_certadd failed: %m\n");
}

/********************************************************************************
 * Description: Delete a VSD cert/key pair from VSD CA
 *
 *     Returns: void
 *******************************************************************************/
void 
ca_certdel (struct connection *vc, 
	    const int         argc, 
	    const char        *argv[])
{
  char *cmd, tmp[128];
  int x;
  
  if (argc != 1)
  {
    io_buffer_store (vc, "-ERR [ca] Incorrect arguments.\n");
    return;
  }
  
  memset (tmp, 0, sizeof(tmp));
#ifdef DEBUG
  srand (time (NULL));
  x = rand () % 1383252;
  sprintf (tmp, "/tmp/vsd%06u", x);
#else
  strcpy (tmp, "/dev/null");
#endif
  
  if (access ("/usr/sbin/vsd-ca_certdel", R_OK | X_OK) == 0)
    cmd = "/usr/sbin/vsd-ca_certdel";
  else if (access ("/usr/local/sbin/vsd-ca_certdel", R_OK | X_OK) == 0)
    cmd = "/usr/local/sbin/vsd-ca_certdel";
  else
    {
      io_buffer_store (vc, "-ERR [ca] Cannot find vsd-ca_certdel.\n");
      return;
    }

  if (run_command (vc, vc->server_root, "root", cmd, argv[0], NULL, tmp))
    io_buffer_store (vc, "-ERR [ca] vsd-ca_certdel failed: %m\n");
}

/********************************************************************************
 * Description: Get a VSD cert/key pair from VSD CA
 *
 *     Returns: void
 *******************************************************************************/
void 
ca_certget (struct connection *vc, 
	    const int         argc, 
	    const char        *argv[])
{
  char crt[128], key[128];
  struct vsd_file_line *fcrt, *fkey, *line;

  if (argc != 1)
  {
    io_buffer_store (vc, "-ERR [ca] Incorrect arguments.\n");
    return;
  }
  
  sprintf (crt, "/etc/vsd/certs/%s.crt", argv[0]);
  if (access (crt, R_OK | X_OK) != 0)
    {
      sprintf (crt, "/usr/local/etc/vsd/certs/%s.crt", argv[0]);
      if (access (crt, R_OK | X_OK) != 0)
	{
	  io_buffer_store (vc, "-ERR [ca] Cannot find certificate.\n");
	  return;
	}
    }
  
  sprintf (key, "/etc/vsd/certs/%s.key", argv[0]);
  if (access (key, R_OK | X_OK) != 0)
    {
      sprintf (key, "/usr/local/etc/vsd/certs/%s.key", argv[0]);
      if (access (key, R_OK | X_OK) != 0)
	{
	  io_buffer_store (vc, "-ERR [ca] Cannot find key.\n");
	  return;
	}
    }
  
  if (vsd_load_file (&fcrt, vc->server_root, crt))
    {
      io_buffer_store (vc, "-ERR [ca] Loading certificate: %m\n");
      return;
    }

  if (vsd_load_file (&fkey, vc->server_root, key))
    {
      io_buffer_store (vc, "-ERR [ca] Loading key: %m\n");
      return;
    }

  for (line = fcrt; line; line=line->next)
    io_buffer_store (vc, "%s\n", line->line);
  
  for (line = fkey; line; line=line->next)
    io_buffer_store (vc, "%s\n", line->line);
}













