/* mod_ca.c - ca module interface.
 */

/* 
 * VSD - Virtual Server Daemon
 * Copyright (c) 2000 - 2001 Idaya Ltd.
 *
 * VSD is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either 
 * version 2 of the License, or (at your option) any later version.
 *
 * VSD is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with VSD - see the COPYING file;  if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA.  
 */

/*
 * See the AUTHORS file for a list of contributors to VSD. 
 * See the ChangeLog files for a list of changes. 
 */

#include <stdio.h>
#include <errno.h>
#include <string.h>

#include "module.h"
#include "mod_ca.h"
#include "ca.h"

static struct command_tag clist[] =
{
  {
    CMD_CA_CERTADD, ADMIN_HOST,
    "Add a VSD cert/key pair to VSD CA",
    "CA_CERTADD", 1, "<cn>",
    "ca_certadd", 2, "<vsdsvr> <cn>"
  },
  {
    CMD_CA_CERTDEL, ADMIN_HOST,
    "Delete a VSD cert/key pair from VSD CA",
    "CA_CERTDEL", 1, "<cn>",
    "ca_certdel", 2, "<vsdsvr> <cn>"
  },
  {
    CMD_CA_CERTGET, ADMIN_HOST,
    "Get a VSD cert/key pair from VSD CA",
    "CA_CERTGET", 1, "<cn>",
    "ca_certget", 2, "<vsdsvr> <cn>"
  },

  {
    0, 0, NULL, NULL, 0, NULL, NULL, 0, NULL
  }
};

/* Return -1 if `cmd' does not exist.
   Return 1 if there is a syntax error.
   Return 0 if everything is cool.  */
int mod_ca_check_cmd (int level, const char *cmd, int argc, char *argv[],
		      struct command_tag **list)
{
  if (cmd == NULL)
    {
      /* Return a pointer to all commands.  */
      *list = clist;
      return 0;
    }

  /* Find command in clist.  */
  *list = module_lookup_command (clist, level, cmd);
  if (*list == NULL)
    return -1;  /* Command does not exist.  */

  /* Check command syntax.  Pretty basic at the moment.  */
#ifdef VSD_LIB
  if (argc < (*list)->vsdadm_args)
    return 1; /* Syntax error.  */
#else
  if (argc < (*list)->vsd_args)
    return 1; /* Syntax error.  */
#endif

  /* Syntax is ok.  */
  return 0;
}

#ifdef VSD_LIB
char *mod_ca_cmd (void *io, int level, const char *cmd,
		  int argc, char **argv)
{
  struct command_tag *list;
  char *status = NULL;

  list = module_lookup_command (clist, level, cmd);
  if (! list)
    return "notexist";

  vsd_set_level (io, list->level);
  switch (list->token)
    {
    case CMD_CA_CERTADD:
      vsd_send_command (io, NULL, 
			"CA_CERTADD %s",
			argv[1] /* cn */); 
      status = vsd_process_reply (io);
      break;
    case CMD_CA_CERTDEL:
      vsd_send_command (io, NULL, 
			"CA_CERTDEL %s",
			argv[1] /* cn */); 
      status = vsd_process_reply (io);
      break;
    case CMD_CA_CERTGET:
      vsd_send_command (io, NULL, 
			"CA_CERTGET %s",
			argv[1] /* cn */); 
      status = vsd_process_reply (io);
      break;
    default:
      status = "mod_ca: command not recognised";
      break;
    }
  return status;
}

#else /* ! VSD_LIB */
/* Return 0 if command processed, 1 if not recognised.  */
int mod_ca_cmd (struct connection *vc,
		int level, const char *command,
		const int argc, const char *argv[], const char *data)
{
  int token;
  struct command_tag *tag;

  tag = module_lookup_command (clist, level, command);
  if (tag == NULL)
    return 1;

  token = tag->token;
  switch (token)
    {
    case CMD_CA_CERTADD:
      ca_certadd (vc, argc, argv);
      break;
    case CMD_CA_CERTDEL:
      ca_certdel (vc, argc, argv);
      break;
    case CMD_CA_CERTGET:
      ca_certget (vc, argc, argv);
      break;
    default:
      return 1;
      break;
    }

  return 0;
}
#endif /* VSD_LIB */

module ca_module =
{
  mod_ca_check_cmd,
  mod_ca_cmd
};




